#ident	"@(#)Files.sh	1.2	92/06/28	JPB"

#	Copyright (c) 1990 UNIX System Laboratories, Inc.
#	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T
#	  All Rights Reserved

#	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
#	UNIX System Laboratories, Inc.
#	The copyright notice above does not evidence any
#	actual or intended publication of such source code.

#ident	"@(#)qt:ID/Files.sh	1.3.1.1"
/usr/lib/tape/tapecntl
#/usr/bin/backup
/dev/rmt/c0s0
/dev/rmt/c0s0n
/dev/rmt/c0s0r
/dev/rmt/c0s0nr
/dev/rmt
/etc/conf/pack.d/qt
/etc/conf/pack.d/qt/Driver.o
/etc/conf/pack.d/qt/space.c
/etc/conf/sdevice.d/qt
/usr/include/sys/qt.h
/usr/vmsys/OBJECTS/tape/Menu.tape
/usr/vmsys/OBJECTS/tape/Text.tape
/usr/vmsys/OBJECTS/tape/Text.fop
/usr/vmsys/OBJECTS/tape/Text.sop
