/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)master:RFS/stubs.c	1.3"

int rf_state;

rfubyte(){ return(0); }
rfuword(){ return(0); }
rsubyte(){ return(0); }
rsuword(){ return(0); }
rcopyin(){ return(0); }
rcopyout(){ return(0); }
rf_vp_preSVR4(){ return(0); }
rf_clock(){ return(0); }
rf_stime(){ return(0); }
rfsys(){ return(0); }
rfc_disable_msg(){ return(0); }
vtord(){ return(0); }
duustat(){ return(0); }
rf_ustat(){ return(0); }
