#!/sbin/sh
#	Copyright (c) 1990 UNIX System Laboratories, Inc.
#	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T
#	  All Rights Reserved

#	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
#	UNIX System Laboratories, Inc.
#	The copyright notice above does not evidence any
#	actual or intended publication of such source code.



#ident	"@(#)oamintf:xx/FACE3.2/admbin/PS/FORM/cspwls.sh	1.1.1.1"

( echo "cs0
cs1
cs2
cs3
cs4
cs5
cs6
cs7
cs8
cs9
cs10
cs11
cs12
cs13
cs14
cs15
cs16
cs17
cs18
cs19
cs20
cs21
cs22
cs23
cs24
cs25
cs26
cs27
cs28
cs29
cs30
cs31
cs32
cs33
cs34
cs35
cs36
cs37
cs38
cs39
cs40
cs41
cs42
cs43
cs44
cs45
cs46
cs47
cs48
cs49
cs50
cs51
cs52
cs53
cs54
cs55
cs56
cs57
cs58
cs59
cs60
cs61
cs62
cs63" ; ls /usr/spool/lp/admins/lp/pwheels ; echo any ) > /usr/tmp/pwlist.$VPID 
