/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)valtools:usage.h	1.1.2.1"

#define OPTMESG \
"where options may include:\n"

#define STDOPTS \
"\t-Q  #quit not allowed\n\t-W width\n\t-d default\n\t-e error\n\t-h help\n\
\t-k pid [-s signo]\n\t-p prompt\n"
