/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)bkrs:rsoper.d/hdrs/rsoper.h	1.3.2.1"

/* defines for options */
#define	cFLAG	0x1
#define	dFLAG	0x2
#define	jFLAG	0x4
#define	mFLAG	0x8
#define	nFLAG	0x10
#define	oFLAG	0x20
#define	rFLAG	0x40
#define	sFLAG	0x80
#define	tFLAG	0x100
#define	uFLAG	0x200
#define	vFLAG	0x400
