/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1988 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)as:i386/systems.h	1.1"
/*
 */

/*
 *	SYSTEMS.H FOR B16
 */

#define DEBUG	0
#define DCODGEN 0
#define M4ON	1
#define FLOAT	0
#define MACRO	0
#define	STATS	0
#define	STATPATH "as_stats.out"
