/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)head.sys:sys/region.h	11.1.4.1"

/* This header file, region.h is kept for source compatibility for those
 * pre-SVR4.0 applications that may include it, but are not regions-based.
 * SVR4.0 memory management is implemented using VM and will not support
 * any application which depends upon regions.
 */
