/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)oamintf:interface/errors.h	1.1.1.2"

#define SUCCESS		0
#define USAGE_ERR	2
#define N_UNIQ_ERR	3
#define N_EXIST_ERR	4
#define EMPTY_ERR	5
#define INV_ERR		6
#define NO_FMLI_ERR	7
#define LOC_ERR		8
#define NOEXPR		9
#define ENV_EXP		10
#define INV_TERM	11
