/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)/usr/src/head/syslog.h.sl 1.1 4.0 12/08/90 3981 AT&T-USL"

#ifndef	_SYSLOG_H
#define	_SYSLOG_H

#include <sys/syslog.h>

#endif /* _SYSLOG_H */
