/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 * Copyright  (c) 1985 AT&T
 *	All Rights Reserved
 *
 */

#ident	"@(#)fmli:inc/ctl.h	1.8"

#define	CTGETVT		1
#define CTGETPID	2
#define CTSETPID	3
#define CTGETPOS	4
#define CTSETPOS	5
#define CTGETSIZ	6
#define CTSETSIZ	7
#define CTGETSTRT	8
#define CTSETSTRT	9
#define CTSETLIM	10
#define CTGETCUR	11
#define CTGETWDW	12
#define CTSETWDW	13
#define CTGETITLE	14
#define CTSETITLE	15
#define CTGETARG	16
#define CTSETARG	17
#define CTSETATTR	18
#define CTSETPARROWS	19
#define CTSETSARROWS	20
#define CTHIDE		21
#define CTSETDIRTY	22
#define CTSETLIFE	23
#define CTISDEST	24
#define CTGETPARMS	25
#define CTSETPARMS	26
#define CTSETSHAPE	27
#define CTSETPAGE	28
#define CTGETLIST	29
#define CTGETINTR	30
#define CTSETINTR	31
#define CTGETONINTR 	32
#define CTSETONINTR 	33
#define CTSETMSG	34
#define CTCLEARWIN	35
#define CTGETDESCRIPTION 36
