Commands are used to take action on objects and text frames.  In AT&T
FACE, objects are forms and menus, and files that are accessed through
the Office. A text frame is used to display the tasks you are working
on.  Commands can be executed from the Commands Menu, from the command
line, or from function keys.  The Frames and Function Key Overview
lists the commands that are available on the function keys.

The Commands Menu is a multi-column menu of the FACE commands. It
provides you with a commands reference list as well as providing you
with a way to issue commands.  The HELP (F1) that is available for
each command on the Commands Menu explains the command that is
highlighted and tells you if the command takes arguments and what
they are.

Executing commands from the Commands Menu:

     1.  Strike the function key labeled CMD-MENU (F7) or type
         cmd-menu on the command line.  The Commands Menu will appear
         and the function key labels will change to: HELP (F1) and
         CANCEL (F6).  
 
     2.	 Navigate in the Commands Menu with the arrow keys. If you
         would like to issue a command to perform an action on the
         current frame that contains a form, menu, file, or file
         folder, or if you would like to issue a command that needs no
         arguments strike ENTER.  The command selected will be issued
         and the Commands Menu will close.

     3.  If you would like to issue a command from the  Commands  Menu,
         but you would like to specify your own command arguments,
         move the cursor to the command you would like to issue and
         strike ctrl-z (depress the CTRL key and strike z simultaneously).
 	 The highlighted command will appear on the command
         line.  Type  additional arguments and strike ENTER to perform the 
         action.

     4.  If you do not wish to issue a command from the Commands Menu,
         strike CANCEL to close the Commands Menu Frame.

Executing commands from the command line:

    1. Strike ctrl-z to see the -> prompt on the command line.

    2. Type the command you want to execute, followed by the arguments
       to the command, if any. 
  
       Note: Striking HELP (F1) after you typed the command on the
             command line will provide help for the command. This
             help will describe the valid arguments to the command.

Executing commands from the function keys F1 - F8:

    1. Some of the commands that are in the commands menu can be
       executed through the function keys that are displayed at the
       bottom of the screen.  The Frames and Function Key Overview
       provides details on the values of the function keys for the
       different types of frames.

    2. To issue a command strike the function key that is labeled with
       the selected command.  The command will be issued on the frame
       that is currently active, or the file or file folder that is
       highlighted in the frame.

    3. If the command requires additional arguments, the -> prompt
       will appear on the command line.  Type the arguments to the
       command and strike ENTER.

