/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1988 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)sdb:inc/CC/errno.h	1.1"
/*ident	"@(#)cfront:incl/errno.h	1.5" */

#ifndef ERRNOH
#define ERRNOH
 
/*
 * Error codes
 */

#include <sys/errno.h>
extern int errno;

#endif
