#	Copyright (c) 1990 UNIX System Laboratories, Inc.
#	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T
#	  All Rights Reserved

#	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
#	UNIX System Laboratories, Inc.
#	The copyright notice above does not evidence any
#	actual or intended publication of such source code.

#ident	"@(#)scsi.in:ID/sd01/Node.sh	1.3"
sd01	rdsk/c0t0d0s0	c	0
sd01	rdsk/c0t0d0s1	c	1
sd01	rdsk/c0t0d0s2	c	2
sd01	rdsk/c0t0d0s3	c	3
sd01	rdsk/c0t0d0s4	c	4
sd01	rdsk/c0t0d0s5	c	5
sd01	rdsk/c0t0d0s6	c	6
sd01	rdsk/c0t0d0sa	c	10
sd01	rdsk/c0t0d0sb	c	11
sd01	rdsk/c0t0d0sc	c	12
sd01	rdsk/c0t0d0sd	c	13
sd01	dsk/c0t0d0s0	b	0
sd01	dsk/c0t0d0s1	b	1
sd01	dsk/c0t0d0s2	b	2
sd01	dsk/c0t0d0s3	b	3
sd01	dsk/c0t0d0s4	b	4
sd01	dsk/c0t0d0s5	b	5
sd01	dsk/c0t0d0s6	b	6
sd01	dsk/c0t0d0sa	b	10
sd01	dsk/c0t0d0sb	b	11
sd01	dsk/c0t0d0sc	b	12
sd01	dsk/c0t0d0sd	b	13
sd01	rdsk/c0t1d0s0	c	16
sd01	rdsk/c0t1d0s1	c	17
sd01	rdsk/c0t1d0s2	c	18
sd01	rdsk/c0t1d0s3	c	19
sd01	rdsk/c0t1d0s4	c	20
sd01	rdsk/c0t1d0s5	c	21
sd01	rdsk/c0t1d0s6	c	22
sd01	rdsk/c0t0d0sa	c	26
sd01	rdsk/c0t0d0sb	c	27
sd01	rdsk/c0t0d0sc	c	28
sd01	rdsk/c0t0d0sd	c	29
sd01	dsk/c0t1d0s0	b	16
sd01	dsk/c0t1d0s1	b	17
sd01	dsk/c0t1d0s2	b	18
sd01	dsk/c0t1d0s3	b	19
sd01	dsk/c0t1d0s4	b	20
sd01	dsk/c0t1d0s5	b	21
sd01	dsk/c0t1d0s6	b	22
sd01	dsk/c0t0d0sa	b	26
sd01	dsk/c0t0d0sb	b	27
sd01	dsk/c0t0d0sc	b	28
sd01	dsk/c0t0d0sd	b	29
