/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1988 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)cscope:common/version.h	1.2"
/*	cscope - interactive C symbol cross-reference
 *
 *	Changing the cross-reference file part of the program version
 *	forces rebuilding of the cross-reference.
 */

#define	FILEVERSION	11	/* cross-reference file format version */
#define	FIXVERSION	".5"	/* feature and bug fix version */
