#	Copyright (c) 1990 UNIX System Laboratories, Inc.
#	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T
#	  All Rights Reserved

#	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
#	UNIX System Laboratories, Inc.
#	The copyright notice above does not evidence any
#	actual or intended publication of such source code.

#ident	"@(#)xl:ID/Files.sh	1.3"

/usr/lib/tape/ftape
/usr/include/sys/ftape.h
/usr/bin/backup
/dev/rmt/f1q80
/dev/rmt/f1q80n
/dev/rmt/f1q80nr
/dev/rmt/f1q80r
/dev/rmt
/etc/conf/pack.d/xl
/etc/conf/pack.d/xl/Driver.o
/etc/conf/pack.d/xl/space.c
/etc/conf/sdevice.d/xl
/usr/vmsys/OBJECTS/ftape/Menu.tape
/usr/vmsys/OBJECTS/ftape/Text.tape
/usr/vmsys/OBJECTS/ftape/Text.fop
/usr/vmsys/OBJECTS/ftape/Text.sop
