/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/



#ident	"@(#)lp:lib/msgs/mclose.c	1.3.2.1"
/* LINTLIBRARY */

#include "lp.h"
#include "msgs.h"

extern MESG	*lp_Md;

int mclose()
{
    MESG	*md = lp_Md;

    lp_Md = 0;

    return(mdisconnect(md));
}
