/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 * Copyright  (c) 1985 AT&T
 *	All Rights Reserved
 *
 */

#ident	"@(#)fmli:inc/attrs.h	1.3"

extern chtype Attr_normal;	/* normal video */
extern chtype Attr_hide;	/* border of non-current window */
extern chtype Attr_highlight;	/* border of current window */
extern chtype Attr_select;	/* attribute of "selector bar" */
extern chtype Attr_show;	/* something visible (errors, etc) */
extern chtype Attr_visible;	/* most annoying thing terminal can do */
extern chtype Attr_underline;	/* attribute of underline */
extern chtype Attr_mark;	/* attribute of "marked" items */

