	/^ramd/s:N:Y:
	/^ansi/s:Y:N:
	/^char/s:Y:N:
	/^clone/s:Y:N:
	/^connld/s:Y:N:
	/^cpyrt/s:Y:N:
	/^gentty/s:Y:N:
	/^i350/s:Y:N:
	/^i410/s:Y:N:
	/^ipc/s:Y:N:
	/^kmacct/s:Y:N:
	/^log/s:Y:N:
	/^mb2stai/s:Y:N:
	/^mb2tsm/s:Y:N:
	/^msg/s:Y:N:
	/^nfa/s:Y:N:
	/^nmi/s:Y:N:
	/^nsxt/s:Y:N:
	/^nxt/s:Y:N:
	/^osm/s:Y:N:
	/^pck/s:Y:N:
	/^ptem/s:Y:N:
	/^ptm/s:Y:N:
	/^pts/s:Y:N:
	/^rmc/s:Y:N:
	/^sem/s:Y:N:
	/^shm/s:Y:N:
	/^sp/s:Y:N:
	/^sxt/s:Y:N:
	/^timod/s:Y:N:
	/^tirdwr/s:Y:N:
	/^ttcompat/s:Y:N:
	/^vx/s:Y:N:
	/^weitek/s:Y:N:
	/^xsd/s:Y:N:
	/^xsem/s:Y:N:
	/^xt/s:Y:N:
