/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1988 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)curses:screen/curses_inc.h	1.2"
/*
 * Internal header file for the library
 *
 */

/* LINTLIBRARY */

#define		CURSES	/* We are internal to curses */

#include	"curses.h"
#include	"term.h"
#include	"curshdr.h"
#include	"unctrl.h"
