#ident	"@(#)oamintf:interface/oam.cmd	1.2"

name=goto
action=NOP

#name=cancel
#action=NOP

#name=cleanup
#action=NOP

name=frm-mgmt
action=NOP

#name=help
#action=NOP

name=next-frm
action=NOP

name=prev-frm
action=NOP

name=unix
action=NOP

#name=update
#action=NOP

