#	Copyright (c) 1990 UNIX System Laboratories, Inc.
#	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T
#	  All Rights Reserved

#	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
#	UNIX System Laboratories, Inc.
#	The copyright notice above does not evidence any
#	actual or intended publication of such source code.

#ident	"@(#)qt:uface/admin.sh	1.3"
name=Tape Operations
action=open menu $VMSYS/OBJECTS/tape/Menu.tape
itemmsg="Move to an item with arrow keys and strike the ENTER key to select"

