/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)libnsl:dial/uucpdefs.c	1.1.1.1"

#include "uucp.h"

/* Configurable parameters */

GLOBAL	char	_ProtoCfg[40]="";	/* protocol string from Config file */

/* Non-configurable parameters */

GLOBAL	int	Ifn, Ofn;
GLOBAL	int 	Sgrades = FALSE;
GLOBAL	int	Debug = 0;
GLOBAL	int	SizeCheck = 0;		/* Ulimit checking supported flag */
GLOBAL	long	RemUlimit = 0;		/* Ulimit of remote if supported */
GLOBAL	int	Restart = 0;		/* checkpoint restart supported flag */
GLOBAL	uid_t	Uid, Euid;		/* user-id and effective-uid */
GLOBAL	long	Ulimit;
GLOBAL	mode_t	Dev_mode;		/* save device mode here */
GLOBAL	char	Progname[NAMESIZE];
GLOBAL	char	Pchar;
GLOBAL	char	Grade = 'Z';
GLOBAL	char	Rmtname[MAXFULLNAME];
GLOBAL	char	JobGrade[MAXBASENAME+1] = { NULLCHAR };
GLOBAL	char	RemSpool[MAXFULLNAME];	/* spool subdirectory for remote system */
GLOBAL	char	User[MAXFULLNAME];
GLOBAL	char	Uucp[NAMESIZE];
GLOBAL	char	Loginuser[NAMESIZE];
GLOBAL	char	Myname[MAXBASENAME+1];
GLOBAL	char	Wrkdir[MAXFULLNAME];
GLOBAL	char	Logfile[MAXFULLNAME];
GLOBAL	char	*Spool = SPOOL;
GLOBAL	char	*Pubdir = PUBDIR;
GLOBAL	char	**Env;

extern int	read();
extern int	write();

GLOBAL	long	Retrytime = 0;
GLOBAL	struct	nstat Nstat;
GLOBAL	char	Dc[50];			/* line name				*/
GLOBAL	int	Seqn;			/* sequence #				*/
GLOBAL	int	Role;
GLOBAL	char	*Bnptr;			/* used when BASENAME macro is expanded */
GLOBAL	char	Jobid[NAMESIZE] = "";	/* Jobid of current C. file */
GLOBAL	int	Uerror;			/* global error code */

GLOBAL	void	(*genbrk)();

GLOBAL	int	Verbose = 0;	/* only for cu and ct to change */

/* used for READANY and READSOME macros */
GLOBAL	struct stat __s_;

/* messages */
GLOBAL char	*Ct_OPEN =	"CAN'T OPEN";
GLOBAL char	*Ct_WRITE =	"CAN'T WRITE";
GLOBAL char	*Ct_READ =	"CAN'T READ";
GLOBAL char	*Ct_CREATE =	"CAN'T CREATE";
GLOBAL char	*Ct_ALLOCATE =	"CAN'T ALLOCATE";
GLOBAL char	*Ct_LOCK =	"CAN'T LOCK";
GLOBAL char	*Ct_STAT =	"CAN'T STAT";
GLOBAL char	*Ct_CHOWN =	"CAN'T CHOWN";
GLOBAL char	*Ct_CHMOD =	"CAN'T CHMOD";
GLOBAL char	*Ct_LINK =	"CAN'T LINK";
GLOBAL char	*Ct_CHDIR =	"CAN'T CHDIR";
GLOBAL char	*Ct_UNLINK =	"CAN'T UNLINK";
GLOBAL char	*Wr_ROLE =	"WRONG ROLE";
GLOBAL char	*Ct_CORRUPT =	"CAN'T MOVE TO CORRUPTDIR";
GLOBAL char	*Ct_CLOSE =	"CAN'T CLOSE";
GLOBAL char	*Ct_FORK =	"CAN'T FORK";
GLOBAL char	*Fl_EXISTS =	"FILE EXISTS";
GLOBAL char	*Ct_BADOWN =	"BAD OWNER/PERMS";

GLOBAL	char *UerrorText[] = {
  /* SS_OK			0 */ "SUCCESSFUL",
  /* SS_NO_DEVICE		1 */ "NO DEVICES AVAILABLE",
  /* SS_TIME_WRONG		2 */ "WRONG TIME TO CALL",
  /* SS_INPROGRESS		3 */ "TALKING",
  /* SS_CONVERSATION		4 */ "CONVERSATION FAILED",
  /* SS_SEQBAD			5 */ "BAD SEQUENCE CHECK",
  /* SS_LOGIN_FAILED		6 */ "LOGIN FAILED",
  /* SS_DIAL_FAILED		7 */ "DIAL FAILED",
  /* SS_BAD_LOG_MCH		8 */ "BAD LOGIN/MACHINE COMBINATION",
  /* SS_LOCKED_DEVICE		9 */ "DEVICE LOCKED",
  /* SS_ASSERT_ERROR		10 */ "ASSERT ERROR",
  /* SS_BADSYSTEM		11 */ "SYSTEM NOT IN Systems FILE",
  /* SS_CANT_ACCESS_DEVICE	12 */ "CAN'T ACCESS DEVICE",
  /* SS_DEVICE_FAILED		13 */ "DEVICE FAILED",
  /* SS_WRONG_MCH		14 */ "WRONG MACHINE NAME",
  /* SS_CALLBACK		15 */ "CALLBACK REQUIRED",
  /* SS_RLOCKED			16 */ "REMOTE HAS A LCK FILE FOR ME",
  /* SS_RUNKNOWN		17 */ "REMOTE DOES NOT KNOW ME",
  /* SS_RLOGIN			18 */ "REMOTE REJECT AFTER LOGIN",
  /* SS_UNKNOWN_RESPONSE	19 */ "REMOTE REJECT, UNKNOWN MESSAGE",
  /* SS_STARTUP			20 */ "STARTUP FAILED",
  /* SS_CHAT_FAILED		21 */ "CALLER SCRIPT FAILED",
  /* SS_CALLBACK_LOOP		22 */ "CALLBACK REQUIRED - LOOP",
};
