/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1988 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)lxprof:cmd/coredefs.h	1.1"
#define	BITCOUNT	8
#define BITMASK		0x1


/* REPORT TYPES */

#define BLANK		0
#define LINE		1
#define LIST		2
#define SUM		3
#define LISTALL		4
