/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _SYS_INO_H
#define _SYS_INO_H

#ident	"@(#)head.sys:sys/ino.h	11.4.3.1"
/*
 * WARNING.  This stub will be removed in System V Release 5.0.
 * Be advised to change any source code using this header file.
 */
#include <sys/fs/s5ino.h>

#endif	/* _SYS_INO_H */
