_m4_dnl_	#ident	"@(#)xcplibx:libx/i386/sys.def	1.1"

_m4_define_(`SYSCALL', ``$'0x7,`$'0')
_m4_define_(`SIGCALL', ``$'0xF,`$'0')

_m4_define_(`ACANCEL', 110)
_m4_define_(`ACCESS', 33)
_m4_define_(`ACCT', 51)
_m4_define_(`ADJTIME', 138)
_m4_define_(`ADVFS', 70)
_m4_define_(`ALARM', 27)
_m4_define_(`BREAK', 17)
_m4_define_(`BRK', 17)
_m4_define_(`BRKBASE', 127)
_m4_define_(`CHDIR', 12)
_m4_define_(`CHMOD', 15)
_m4_define_(`CHOWN', 16)
_m4_define_(`CHROOT', 61)
_m4_define_(`CHSIZE', 2600)
_m4_define_(`CLOSE', 6)
_m4_define_(`CREAT', 8)
_m4_define_(`CREATSEM', 552)
_m4_define_(`CXENIX', 40)
_m4_define_(`DUP', 41)
_m4_define_(`EVSYS', 101)
_m4_define_(`EVTRAPRET', 102)
_m4_define_(`EXEC', 11)
_m4_define_(`EXECE', 59)
_m4_define_(`EXECSEG', 8488)
_m4_define_(`EXIT', 1)
_m4_define_(`FCHDIR', 120)
_m4_define_(`FCHMOD', 93)
_m4_define_(`FCHOWN', 94)
_m4_define_(`FCNTL', 62)
_m4_define_(`FMOUNT', 105)
_m4_define_(`FORK', 2)
_m4_define_(`FSTAT', 28)
_m4_define_(`FSTATF', 83)
_m4_define_(`FSTATFS', 38)
_m4_define_(`FSTATVFS', 104)
_m4_define_(`FSYNC', 58)
_m4_define_(`FTIME', 2856)
_m4_define_(`FUNMOUNT', 106)
_m4_define_(`FXSTAT', 125)
_m4_define_(`GETDENTS', 81)
_m4_define_(`GETGID', 47)
_m4_define_(`GETGROUPS', 92)
_m4_define_(`GETMSG', 85)
_m4_define_(`GETPAGESIZE', 118)
_m4_define_(`GETPID', 20)
_m4_define_(`GETPMSG', 132)
_m4_define_(`GETRLIMIT', 129)
_m4_define_(`GETUID', 24)
_m4_define_(`GTTY', 32)
_m4_define_(`IOCTL', 54)
_m4_define_(`KILL', 37)
_m4_define_(`LCHOWN', 130)
_m4_define_(`LINK', 9)
_m4_define_(`LOCK', 45)
_m4_define_(`LSEEK', 19)
_m4_define_(`LSTAT', 88)
_m4_define_(`LXSTAT', 124)
_m4_define_(`MCTL', 113)
_m4_define_(`MEMCNTL', 131)
_m4_define_(`MINCORE', 114)
_m4_define_(`MKDIR', 80)
_m4_define_(`MKNOD', 14)
_m4_define_(`MMAP', 115)
_m4_define_(`MOUNT', 21)
_m4_define_(`MPROTECT', 116)
_m4_define_(`MSGSYS', 49)
_m4_define_(`MUNMAP', 117)
_m4_define_(`NAP', 3112)
_m4_define_(`NBWAITSEM', 1576)
_m4_define_(`NICE', 34)
_m4_define_(`NUNAME', 135)
_m4_define_(`OPEN', 5)
_m4_define_(`OPENSEM', 808)
_m4_define_(`PAUSE', 29)
_m4_define_(`PIPE', 42)
_m4_define_(`PLOCK', 45)
_m4_define_(`POLL', 87)
_m4_define_(`PRIOCNTLSET', 112)
_m4_define_(`PROCTL', 8232)
_m4_define_(`PROF', 44)
_m4_define_(`PTRACE', 26)
_m4_define_(`PUTMSG', 86)
_m4_define_(`PUTPMSG', 133)
_m4_define_(`RDCHK', 1832)
_m4_define_(`RDEBUG', 76)
_m4_define_(`READ', 3)
_m4_define_(`READLINK', 90)
_m4_define_(`READV', 121)
_m4_define_(`RENAME', 134)
_m4_define_(`RFSTART', 74)
_m4_define_(`RFSTOP', 77)
_m4_define_(`RFSYS', 78)
_m4_define_(`RMDIR', 79)
_m4_define_(`RMOUNT', 72)
_m4_define_(`RUMOUNT', 73)
_m4_define_(`SDENTER', 3880)
_m4_define_(`SDFREE', 3624)
_m4_define_(`SDGET', 3368)
_m4_define_(`SDGETV', 4392)
_m4_define_(`SDLEAVE', 4136)
_m4_define_(`SDWAITV', 4648)
_m4_define_(`SEEK', 19)
_m4_define_(`SEMSYS', 53)
_m4_define_(`SETEGID', 140)
_m4_define_(`SETEUID', 141)
_m4_define_(`SETGID', 46)
_m4_define_(`SETGROUPS', 91)
_m4_define_(`SETPGRP', 39)
_m4_define_(`SETRLIMIT', 128)
_m4_define_(`SETSID', 39)
_m4_define_(`SETUID', 23)
_m4_define_(`SHMSYS', 52)
_m4_define_(`SIGACTION', 98)
_m4_define_(`SIGALTSTACK', 97)
_m4_define_(`SIGNAL', 48)
_m4_define_(`SIGPENDING', 99)
_m4_define_(`SIGPROCMASK', 95)
_m4_define_(`SIGSEM', 1064)
_m4_define_(`SIGSENDSET', 108)
_m4_define_(`SIGSUSPEND', 96)
_m4_define_(`STAT', 18)
_m4_define_(`STATF', 82)
_m4_define_(`STATFS', 35)
_m4_define_(`STATVFS', 103)
_m4_define_(`STIME', 25)
_m4_define_(`STTY', 31)
_m4_define_(`SYMLINK', 89)
_m4_define_(`SYNC', 36)
_m4_define_(`SYS3B', 50)
_m4_define_(`SYSACCT', 51)
_m4_define_(`SYSCONFIG',  137)
_m4_define_(`SYSFS', 84)
_m4_define_(`SYSI86',  50)
_m4_define_(`SYSTEMINFO',  139)
_m4_define_(`TIME', 13)
_m4_define_(`TIMES', 43)
_m4_define_(`UADMIN', 55)
_m4_define_(`UCONTEXT', 100)
_m4_define_(`ULIMIT', 63)
_m4_define_(`UMASK', 60)
_m4_define_(`UMOUNT', 22)
_m4_define_(`UNADVFS', 71)
_m4_define_(`UNEXECSEG', 8744)
_m4_define_(`UNLINK', 10)
_m4_define_(`UTIME', 30)
_m4_define_(`UTSSYS', 57)
_m4_define_(`VFORK', 119)
_m4_define_(`WAIT', 7)
_m4_define_(`WAITID', 107)
_m4_define_(`WAITSEM', 1320)
_m4_define_(`WRITE', 4)
_m4_define_(`WRITEV', 122)
_m4_define_(`XLOCKING', 296)
_m4_define_(`XMKNOD', 126)
_m4_define_(`XSTAT', 123)
