/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _SYS_FILSYS_H
#define _SYS_FILSYS_H

#ident	"@(#)head.sys:sys/filsys.h	11.3.3.1"
/* WARNING!!!  This stub will be removed for release 4.0. Please 
*  be advised to change any source code using this header file */
#include <sys/fs/s5filsys.h>

#endif	/* _SYS_FILSYS_H */
