/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/


#ident	"@(#)ksh:include/flags.h	1.1.3.1"
#define R_FLAG	01
#define I_FLAG	02
#define U_FLAG	04
#define L_FLAG	010
#define Z_FLAG	020
#define W_FLAG	040
#define S_FLAG	0100
#define T_FLAG	0200
#define M_FLAG	0400
#define X_FLAG	01000
#define N_FLAG	02000
#define F_FLAG	04000
#define G_FLAG	010000
#define P_FLAG	020000
#define V_FLAG	040000
#define E_FLAG	0100000
#define	A_FLAG	0200000
#define B_FLAG	0400000
#define	C_FLAG	01000000
#define	D_FLAG	02000000
#define	H_FLAG	04000000
