/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)bkrs:intftools.d/hdrs/cperrs.h	1.1.2.1"

/* "Option \"%c\" is invalid.\n" */
#define ERROR0	0

/* "Illegal number of arguments.\n" */
#define ERROR1	1

/* "Invalid option or argument \"%s\".\n" */
#define ERROR2	2

/* "Cannot open file %s.\n" */
#define ERROR3	3

/* "\"%s\" is not a valid field separator.\n" */
#define ERROR4	4

/* "Invalid invocation argument \"%s\".\n" */
#define ERROR5	5
