/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1988 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)stitam:llib-lwind.c	1.1"
/*LINTLIBRARY*/
#include "tam.h"
#include "wind.h"
#include "track.h"
#include "sys/window.h"

int	wind (type,height,width,flags,pfont) int type,height,width; short flags; char *pfont[]; {return 0;}
do_w_new(height,width)
void	int height,width; {}
