/^NBUF/c\
NBUF		40	0	40
/^NINODE/c\
NINODE		50	0	50
/^NS5INODE/c\
NS5INODE	50	0	50
/^NFILE/c\
NFILE		50	0	50
/^NPROC/c\
NPROC		30	0	30
/^NCLIST/c\
NCLIST		2	0	2
/^NHBUF/c\
NHBUF		32	0	32
/^NPBUF/c\
NPBUF		20	0	20
/^NAUTOUP/c\
NAUTOUP		300	0	300
/^NREGION/c\
NREGION		100	0	100
/^SPTMAP/c\
SPTMAP		70	0	70
/^MSGSEG/c\
MSGSEG		128	0	128
/^PUTBUFSZ/c\
PUTBUFSZ	1024	0	1024
/^KDBSYMSIZE/c\
KDBSYMSIZE	100	0	100000
/^DO387CR3/c\
DO387CR3		0	0	2
