#	Copyright (c) 1990 UNIX System Laboratories, Inc.
#	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T
#	  All Rights Reserved

#	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
#	UNIX System Laboratories, Inc.
#	The copyright notice above does not evidence any
#	actual or intended publication of such source code.

#ident	"@(#)scsi.in:ID/sc01/Node.sh	1.3"
sc01	rcdrom/cd0	c	0
sc01	rcdrom/cd1	c	1
sc01	rcdrom/cd2	c	2
sc01	rcdrom/cd3	c	3
sc01	rcdrom/cd4	c	4
sc01	rcdrom/cd5	c	5
sc01	rcdrom/cd6	c	6
sc01	cdrom/cd0	b	0
sc01	cdrom/cd1	b	1
sc01	cdrom/cd2	b	2
sc01	cdrom/cd3	b	3
sc01	cdrom/cd4	b	4
sc01	cdrom/cd5	b	5
sc01	cdrom/cd6	b	6
