/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/


/*      Portions Copyright (c) 1988, Sun Microsystems, Inc.     */ 
/*      All Rights Reserved.                                    */ 
 
#ident	"@(#)ucbprt:hdr/had.h	1.1.1.1"

# define HADA had[('a'-'a')]
# define HADB had[('b'-'a')]
# define HADC had[('c'-'a')]
# define HADD had[('d'-'a')]
# define HADE had[('e'-'a')]
# define HADF had[('f'-'a')]
# define HADG had[('g'-'a')]
# define HADH had[('h'-'a')]
# define HADI had[('i'-'a')]
# define HADJ had[('j'-'a')]
# define HADK had[('k'-'a')]
# define HADL had[('l'-'a')]
# define HADM had[('m'-'a')]
# define HADN had[('n'-'a')]
# define HADO had[('o'-'a')]
# define HADP had[('p'-'a')]
# define HADQ had[('q'-'a')]
# define HADR had[('r'-'a')]
# define HADS had[('s'-'a')]
# define HADT had[('t'-'a')]
# define HADU had[('u'-'a')]
# define HADV had[('v'-'a')]
# define HADW had[('w'-'a')]
# define HADX had[('x'-'a')]
# define HADY had[('y'-'a')]
# define HADZ had[('z'-'a')]
