/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/


#ident	"@(#)mbus:uts/i386/master.d/bps/stubs.c	1.2"

bps_open()
{
	return(-1);
}
bps_close(){}
bps_get_opt(){}
bps_get_wcval(){}
bps_get_val(){}
bps_get_integer(){}
bps_get_range(){}
bps_get_socket(){}

