/*	Copyright (c) 1990 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ident	"@(#)/usr/src/head/rpcsvc/spray.h.sl 1.1 4.0 12/08/90 40579 AT&T-USL"

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*	PROPRIETARY NOTICE (Combined)
*
* This source code is unpublished proprietary information
* constituting, or derived under license from AT&T's UNIX(r) System V.
* In addition, portions of such source code were derived from Berkeley
* 4.3 BSD under license from the Regents of the University of
* California.
*
*
*
*	Copyright Notice 
*
* Notice of copyright on this source code product does not indicate 
*  publication.
*
*	(c) 1986,1987,1988.1989  Sun Microsystems, Inc
*	(c) 1983,1984,1985,1986,1987,1988,1989  AT&T.
*          All rights reserved.
*/ 
/*	@(#)spray.h 1.4 89/03/24 SMI	*/

/*
 * spray.h
 */

/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#define SPRAYMAX 8845

struct spraytimeval {
	u_int sec;
	u_int usec;
};
typedef struct spraytimeval spraytimeval;
bool_t xdr_spraytimeval();

struct spraycumul {
	u_int counter;
	spraytimeval clock;
};
typedef struct spraycumul spraycumul;
bool_t xdr_spraycumul();

typedef struct {
	u_int sprayarr_len;
	char *sprayarr_val;
} sprayarr;
bool_t xdr_sprayarr();

#define SPRAYPROG ((u_long)100012)
#define SPRAYVERS ((u_long)1)
#define SPRAYPROC_SPRAY ((u_long)1)
extern void *sprayproc_spray_1();
#define SPRAYPROC_GET ((u_long)2)
extern spraycumul *sprayproc_get_1();
#define SPRAYPROC_CLEAR ((u_long)3)
extern void *sprayproc_clear_1();
